/// Errors during lexical analysis.
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
pub enum LexerError {
    /// An invalid character literal was encountered.
    InvalidChar,
    /// An invalid keyword literal was encountered.
    InvalidKeyword,
    /// An invalid number literal was encountered.
    InvalidNumber,
    /// An invalid token was encountered.
    InvalidToken,
    /// An unclosed string was encountered.
    UnclosedString,
    /// Invalid UTF-8 sequence was encountered.
    Utf8Error(std::str::Utf8Error),
}

impl From<std::str::Utf8Error> for LexerError {
    fn from(err: std::str::Utf8Error) -> Self {
        Self::Utf8Error(err)
    }
}

impl std::error::Error for LexerError {}

#[cfg(not(tarpaulin_include))]
impl std::fmt::Display for LexerError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            LexerError::InvalidChar => write!(f, "An invalid character literal was encountered"),
            LexerError::InvalidKeyword => write!(f, "An invalid keyword literal was encountered"),
            LexerError::InvalidNumber => write!(f, "An invalid number literal was encountered"),
            LexerError::InvalidToken => write!(f, "An invalid token was encountered"),
            LexerError::UnclosedString => write!(f, "An unclosed string was encountered"),
            LexerError::Utf8Error(err) => write!(f, "{err}"),
        }
    }
}
